@@ -6,11 +6,17 @@ import requests
 import warnings
 from collections import OrderedDict
 import pandas as pd
-from tqdm import tqdm
 from datetime import datetime, timedelta
-from typing import List
-from ..objects import RawBar
+from typing import List, Callable
+from urllib.parse import quote
+
+from ..objects import RawBar, Event
+from ..enum import Freq
+from .base import freq_inv
 from ..utils.kline_generator import bar_end_time
+from ..analyze import CzscTrader, KlineGenerator
+from ..signals import get_default_signals
+
 
 url = "https://dataapi.joinquant.com/apis"
 home_path = os.path.expanduser("~")
@@ -20,6 +26,10 @@ file_token = os.path.join(home_path, "jq.token")
 freq_convert = {"1min": "1m", "5min": '5m', '15min': '15m',
                 "30min": "30m", "60min": '60m', "D": "1d", "W": '1w', "M": "1M"}
 
+freq_map = {'1min': Freq.F1, '5min': Freq.F5, '15min': Freq.F15, '30min': Freq.F30,
+            '60min': Freq.F60, 'D': Freq.D, 'W': Freq.W, 'M': Freq.M}
+
+
 def set_token(jq_mob, jq_pwd):
     """
 
@@ -43,8 +53,8 @@ def get_token():
 
     body = {
         "method": "get_current_token",
-        "mob": jq_mob,  # mob是申请JQData时所填写的手机号
-        "pwd": jq_pwd,  # Password为聚宽官网登录密码，新申请用户默认为手机号后6位
+        "mob": jq_mob,          # mob是申请JQData时所填写的手机号
+        "pwd": quote(jq_pwd),   # Password为聚宽官网登录密码，新申请用户默认为手机号后6位
     }
     response = requests.post(url, data=json.dumps(body))
     token = response.text
@@ -59,6 +69,7 @@ def text2df(text):
     df = pd.DataFrame(rows[1:], columns=rows[0])
     return df
 
+
 def get_query_count() -> int:
     """获取查询剩余条数
     https://dataapi.joinquant.com/docs#get_query_count---%E8%8E%B7%E5%8F%96%E6%9F%A5%E8%AF%A2%E5%89%A9%E4%BD%99%E6%9D%A1%E6%95%B0
@@ -72,6 +83,7 @@ def get_query_count() -> int:
     r = requests.post(url, data=json.dumps(data))
     return int(r.text)
 
+
 def get_concepts():
     """获取概念列表
 
@@ -87,6 +99,7 @@ def get_concepts():
     df = text2df(r.text)
     return df
 
+
 def get_concept_stocks(symbol, date=None):
     """获取概念成份股
 
@@ -118,6 +131,7 @@ def get_concept_stocks(symbol, date=None):
     r = requests.post(url, data=json.dumps(data))
     return r.text.split('\n')
 
+
 def get_index_stocks(symbol, date=None):
     """获取指数成份股
 
@@ -149,6 +163,7 @@ def get_index_stocks(symbol, date=None):
     r = requests.post(url, data=json.dumps(data))
     return r.text.split('\n')
 
+
 def get_industry(symbol):
     """
     https://www.joinquant.com/help/api/help#JQDataHttp:get_industry-%E6%9F%A5%E8%AF%A2%E8%82%A1%E7%A5%A8%E6%89%80%E5%B1%9E%E8%A1%8C%E4%B8%9A
@@ -180,25 +195,6 @@ def get_industry(symbol):
     }
     return res
 
-def get_stock_industry():
-    df = get_all_securities("stock")
-    rows = df.to_dict('records')
-    results = []
-    for i, row in tqdm(rows):
-        try:
-            res = dict(row)
-            res.update(get_industry(res['code']))
-            results.append(res)
-        except Exception as e:
-            print("get industry fail on {}: {}".format(row['code'], e))
-    stock_ind = pd.DataFrame(results)
-    stock_ind.rename({'display_name': '股票名称', 'start_date': '上市日期'}, axis=1, inplace=True)
-    cols = ['股票代码', '股票名称', '上市日期', '证监会行业代码', '证监会行业名称',
-            '聚宽一级行业代码', '聚宽一级行业名称', '聚宽二级行业代码', '聚宽二级行业名称', '申万一级行业代码',
-            '申万一级行业名称', '申万二级行业代码', '申万二级行业名称', '申万三级行业代码', '申万三级行业名称']
-    stock_ind = stock_ind[cols]
-    return stock_ind
-
 
 def get_all_securities(code, date=None) -> pd.DataFrame:
     """
@@ -228,6 +224,7 @@ def get_all_securities(code, date=None) -> pd.DataFrame:
     r = requests.post(url, data=json.dumps(data))
     return text2df(r.text)
 
+
 def get_kline(symbol: str, end_date: [datetime, str], freq: str,
               start_date: [datetime, str] = None, count=None, fq: bool = True) -> List[RawBar]:
     """获取K线数据
@@ -280,7 +277,9 @@ def get_kline(symbol: str, end_date: [datetime, str], freq: str,
 
     r = requests.post(url, data=json.dumps(data))
     rows = [x.split(",") for x in r.text.strip().split('\n')][1:]
+
     bars = []
+    i = -1
     for row in rows:
         # row = ['date', 'open', 'close', 'high', 'low', 'volume', 'money']
         dt = pd.to_datetime(row[0])
@@ -288,7 +287,8 @@ def get_kline(symbol: str, end_date: [datetime, str], freq: str,
             dt = dt.replace(hour=0, minute=0, second=0, microsecond=0)
 
         if int(row[5]) > 0:
-            bars.append(RawBar(symbol=symbol, dt=dt,
+            i += 1
+            bars.append(RawBar(symbol=symbol, dt=dt, id=i, freq=freq_map[freq],
                                open=round(float(row[1]), 2),
                                close=round(float(row[2]), 2),
                                high=round(float(row[3]), 2),
@@ -317,6 +317,9 @@ def get_kline_period(symbol: str, start_date: [datetime, str],
     start_date = pd.to_datetime(start_date)
     end_date = pd.to_datetime(end_date)
 
+    if (end_date - start_date).days * 5 / 7 > 1000:
+        warnings.warn(f"{end_date.date()} - {start_date.date()} 超过1000个交易日，K线获取可能失败，返回为0")
+
     data = {
         "method": "get_price_period",
         "token": get_token(),
@@ -331,6 +334,7 @@ def get_kline_period(symbol: str, start_date: [datetime, str],
     r = requests.post(url, data=json.dumps(data))
     rows = [x.split(",") for x in r.text.strip().split('\n')][1:]
     bars = []
+    i = -1
     for row in rows:
         # row = ['date', 'open', 'close', 'high', 'low', 'volume', 'money']
         dt = pd.to_datetime(row[0])
@@ -338,7 +342,8 @@ def get_kline_period(symbol: str, start_date: [datetime, str],
             dt = dt.replace(hour=0, minute=0, second=0, microsecond=0)
 
         if int(row[5]) > 0:
-            bars.append(RawBar(symbol=symbol, dt=dt,
+            i += 1
+            bars.append(RawBar(symbol=symbol, dt=dt, id=i, freq=freq_map[freq],
                                open=round(float(row[1]), 2),
                                close=round(float(row[2]), 2),
                                high=round(float(row[3]), 2),
@@ -388,6 +393,7 @@ def get_fundamental(table: str, symbol: str, date: str, columns: str = "") -> di
     except:
         return {}
 
+
 def run_query(table: str, conditions: str, columns=None, count=1):
     """模拟JQDataSDK的run_query方法
 
@@ -471,4 +477,45 @@ def get_share_basic(symbol):
         msg += k + "：{} | {} | {} | {}\n".format(*[f10['{}{}'.format(year, k)] for year in ['2017', '2018', '2019', '2020']])
 
     f10['msg'] = msg
-    return f10
\ No newline at end of file
+    return f10
+
+
+class JqCzscTrader(CzscTrader):
+    def __init__(self, symbol, op_freq: Freq = None, max_count=2000, end_date=None,
+                 get_signals: Callable = get_default_signals,
+                 events: List[Event] = None):
+        self.symbol = symbol
+        if end_date:
+            self.end_date = pd.to_datetime(end_date)
+        else:
+            self.end_date = datetime.now()
+        self.max_count = max_count
+        kg = KlineGenerator(max_count=max_count*2)
+        for freq in kg.freqs:
+            bars = get_kline(symbol, end_date=self.end_date, freq=freq_inv[freq], count=max_count)
+            kg.init_kline(freq, bars)
+        super(JqCzscTrader, self).__init__(op_freq=op_freq, kg=kg, get_signals=get_signals, events=events)
+
+    def update_factors(self):
+        """更新K线数据到最新状态"""
+        bars = get_kline_period(symbol=self.symbol, start_date=self.end_dt, end_date=datetime.now(), freq="1min")
+        if not bars or bars[-1].dt <= self.end_dt:
+            return
+        for bar in bars:
+            self.check_operate(bar)
+
+    def forward(self, n: int = 3):
+        """向前推进N天"""
+        ed = self.end_dt + timedelta(days=n)
+        if ed > datetime.now():
+            print(f"{ed} > {datetime.now()}，无法继续推进")
+            return
+
+        bars = get_kline_period(symbol=self.symbol, start_date=self.end_dt, end_date=ed, freq="1min")
+        if not bars:
+            print(f"{self.end_dt} ~ {ed} 没有交易数据")
+            return
+
+        for bar in bars:
+            self.check_operate(bar)
+
