@@ -22,7 +22,7 @@ def SMA(close: np.array, timeperiod=5):
         else:
             seq = close[i - timeperiod + 1: i + 1]
         res.append(seq.mean())
-    return np.array(res, dtype=np.double)
+    return np.array(res, dtype=np.double).round(4)
 
 def EMA(close: np.array, timeperiod=5):
     """
@@ -41,7 +41,7 @@ def EMA(close: np.array, timeperiod=5):
         else:
             ema = (2 * close[i] + res[i-1] * (timeperiod-1)) / (timeperiod+1)
             res.append(ema)
-    return np.array(res, dtype=np.double)
+    return np.array(res, dtype=np.double).round(4)
 
 def MACD(close: np.array, fastperiod=12, slowperiod=26, signalperiod=9):
     """MACD 异同移动平均线
@@ -63,7 +63,7 @@ def MACD(close: np.array, fastperiod=12, slowperiod=26, signalperiod=9):
     diff = ema12 - ema26
     dea = EMA(diff, timeperiod=signalperiod)
     macd = (diff - dea) * 2
-    return diff, dea, macd
+    return diff.round(4), dea.round(4), macd.round(4)
 
 def KDJ(close: np.array, high: np.array, low: np.array):
     """
@@ -108,7 +108,7 @@ def KDJ(close: np.array, high: np.array, low: np.array):
     k = np.array(k, dtype=np.double)
     d = np.array(d, dtype=np.double)
     j = np.array(j, dtype=np.double)
-    return k, d, j
+    return k.round(4), d.round(4), j.round(4)
 
 def RSQ(close: [np.array, list]) -> float:
     """拟合优度 R SQuare
